/*
 * FitnessFunction.java
 *
 * Created on 14 de octubre de 2005, 9:45
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package FitnessFunctions;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import java.util.Vector;

/**
 *
 * @author carlos
 */
public abstract class FitnessFunction {
  
  /** Number of fitness evaluations. */
  protected long numEvaluations;
  
  /** This function assings a double value to the Chormosome c.
   * Its fitness may depend on the remainder chromosomes. */
  public abstract double fitness(Chromosome c, Vector remainder);
  public abstract double fitness(Chromosome c);
  
  /** This function returns the fitness of a chromosome n, which is neighbor of the chromosome o.
   * Its fitness may be calculated considering the diferences between both chromosomes, which uses to be faster. */
  public double fitnessNeighbor(Chromosome n, Chromosome o){
    return this.fitness(n);
  }
  public double fitnessNeighbor(Chromosome n, Chromosome o, Vector changes){
    return this.fitnessNeighbor(n, o);
  }
  public double fitnessNeighbor(Chromosome n, Chromosome o, int change){
    Vector changes = new Vector();
    changes.add(new Integer(change));
    return this.fitnessNeighbor(n, o, changes);
  }
  
  /** This function returns a number greater than 0 if fitness f1 is better than f2,
   * less than 0 if c2 es better than c1, and 0 if both are equivalent. */
  public abstract double compare(double f1, double f2);
  
  /** This function returns the number of fitness evaluations */
  public long getNumEvaluations(){
    return this.numEvaluations;
  }
  
  /** This function increments the number of evaluations */
  public long incrementNumEvaluations(){
    this.numEvaluations++;
    return (this.numEvaluations);
  }
  
  
  public long incrementNumEvaluations(long increment){
    this.numEvaluations += increment;
    return (this.numEvaluations);
  }
  
  
  public long decrementNumEvaluations(){
    this.numEvaluations--;
    return (this.numEvaluations);
  }
  
  
  /** Resets the number of evaluations */
  public void resetNumEvaluations(){
    this.numEvaluations = 0;
  }
  
  /** returns a name for the funcion*/
  public abstract String getName();
  
  /** return the dimension of the function*/
  public abstract int getDimension();
  
  /** return the worst fitnessValue*/
  public abstract double getWorstFitness();
}
