/*
 * BohachevskyFitnessFunction.java
 *
 * Created on 25 de julio de 2005, 20:25
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package FitnessFunctions.RealFF;
import geneticalgorithms.ChromosomeRepresentations.RealChromosome;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import java.util.Vector;


/**
 *
 * @author carlos
 */
public class BohachevskyFitnessFunction extends RealFitnessFunction{
    
    /** Creates a new instance of BohachevskyFitnessFunction */
    public BohachevskyFitnessFunction(double minGenValue, double maxGenValue) {
        super(minGenValue, maxGenValue);
    }
    
    public double compare(double f1, double f2){
        return (f2 - f1);
    }
    
    public double getWorstFitness(){
        return Double.POSITIVE_INFINITY;
    }
    
    public double fitness(Chromosome c){
        
        this.incrementNumEvaluations();
        RealChromosome rc = (RealChromosome) c;
        int dimension = rc.getNumGenes();
        int i;
        double currentGen;
        double nextGen;
        double sum = 0;
        
        currentGen = rc.getGen(0);
        
        for (i = 1; i < dimension; i++){
            nextGen = rc.getGen(i);
            sum += currentGen * currentGen + 2 * nextGen * nextGen;
            sum += -0.3 * Math.cos(3 * Math.PI * currentGen);
            sum += -0.4 * Math.cos(4 * Math.PI * nextGen) + 0.7;
            currentGen = nextGen;
        }
        
        return (sum);
    }
    
    public double fitness(Chromosome c, Vector remainder){
        return (this.fitness(c));
    }
    
    /** returns a name for the funcion*/
    public String getName(){
        return "Boh";
    }
    
    /** return the dimension of the function*/
    public int getDimension(){
        return -1;
    }
}
