package FitnessFunctions.RealFF;

import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import geneticalgorithms.ChromosomeRepresentations.RealChromosome;

/**
 * <p>Título: Genetic Algorithms</p>
 * <p>Descripción: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Empresa: </p>
 * @author Carlos García Martínez
 * @version 1.0
 */

import java.util.Vector;
import FitnessFunctions.FitnessFunction;

/** This class defines the common behavior of every Fitness Function*/

public abstract class RealFitnessFunction extends FitnessFunction {
    
    protected double minGenValue;
    protected double maxGenValue;
    protected double minFitnessValue;
    protected double maxFitnessValue;
    
    //Useful when there are different bounds for each decision variable
    protected int numBounds;
    protected double[] minGenValues;
    protected double[] maxGenValues;
    
    /** Constructor
     *	This constructor is interesting when there are different bounds for each decision variable.
     */
    public RealFitnessFunction(int numGenes, double initMinGenValue, double initMaxGenValue,
            double minFitnessValue, double maxFitnessValue){
        this.numEvaluations = 0;
        this.minGenValues = new double[numGenes];
        this.maxGenValues = new double[numGenes];
        this.minFitnessValue = minFitnessValue;
        this.maxFitnessValue = maxFitnessValue;
        
        int i;
        this.numBounds = numGenes;
        
        for (i = 0; i < numGenes; i++){
            minGenValues[i] = initMinGenValue;
            maxGenValues[i] = initMaxGenValue;
        }
    }
    
    public RealFitnessFunction(double minGenValue, double maxGenValue){
        this.numBounds = 0;
        this.numEvaluations = 0;
        this.minGenValue = minGenValue;
        this.maxGenValue = maxGenValue;
        this.minFitnessValue = 0.0;
        this.maxFitnessValue = Double.MAX_VALUE;
    }
    
    /** Another constructor */
    public RealFitnessFunction(double minGenValue, double maxGenValue, double minFitnessValue, double maxFitnessValue){
        
        this(minGenValue, maxGenValue);
        this.numBounds = 0;
        this.minFitnessValue = minFitnessValue;
        this.maxFitnessValue = maxFitnessValue;
    }
    
    /** Set the ith decision variable bounds. Valid only if */
    
    /** This function tests if a value of a gen is a valid one (if it is in a valid interval).
     * It returns the closest valid value. */
    public double testValue(double value, int index){
        
        if (this.numBounds == 0){
            
            if (value > minGenValue){
                
                if (value < maxGenValue)
                    return value;
                else
                    return maxGenValue;
            }
            
            else
                return minGenValue;
        }
        
        else{
            
            if (value > minGenValues[index]){
                
                if (value < maxGenValues[index])
                    return value;
                else
                    return maxGenValues[index];
            }
            
            else
                return minGenValues[index];
        }
    }
    
    public double getMinGenValue(int index){
        
        if (this.numBounds == 0)
            return this.minGenValue;
        else
            return this.minGenValues[index];
    }
    
    public double getMaxGenValue(int index){
        
        if (this.numBounds == 0)
            return this.maxGenValue;
        else
            return this.maxGenValues[index];
    }
    
    public double getMaxFitnessValue(){
        return this.maxFitnessValue;
    }
    
    public double getMinFitnessValue(){
        return this.minFitnessValue;
    }
    
    public void setIthBound(int index, double minValue, double maxValue){
        
        if (this.numBounds > index){
            this.minGenValues[index] = minValue;
            this.maxGenValues[index] = maxValue;
        }
    }
}
