/*
 * RotatedRastriginFitnessFunction.java
 *
 * Created on 25 de julio de 2005, 19:12
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package FitnessFunctions.RealFF;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import geneticalgorithms.ChromosomeRepresentations.RealChromosome;
import java.util.Vector;


/**
 *
 * @author carlos
 */
public class RotatedRastriginFitnessFunction extends RealFitnessFunction{
    
    double [][]A;
    RealFitnessFunction rastriginFunction;
    
    /** Creates a new instance of RotatedRastriginFitnessFunction */
    public RotatedRastriginFitnessFunction(int dimension, double minGenValue, double maxGenValue) {
        super(minGenValue, maxGenValue);
        int i, j;
        
        rastriginFunction = new RastriginFitnessFunction(10, 2 * Math.PI, minGenValue, maxGenValue);
        this.A = new double[dimension][dimension];
        
        for (i = 0; i < dimension; i++){
            
            for (j = 0; j < dimension; j++){
                
                if (i == j)
                    this.A[i][j] = 4.0 / 5.0;
                
                else if (Math.abs(i - j) == 1)
                    this.A[i][j] = Math.pow(-1.0, j) * 3.0 / 5.0;
                
                else
                    this.A[i][j] = 0;
            }
        }
    }
    
    public double compare(double f1, double f2){
        return (f2 - f1);
    }
    
    public double getWorstFitness(){
        return Double.POSITIVE_INFINITY;
    }
    
    public double fitness(Chromosome c){
        
        this.incrementNumEvaluations();
        RealChromosome rc = (RealChromosome) c.clone();
        int dimension = rc.getNumGenes();
        int i, j;
        double sum;
        
        for (i = 0; i < dimension; i++){
            
            sum = 0;
            
            for (j = 0; j < dimension; j++){
                
                sum += this.A[i][j] * rc.getGen(j);
            }
            
            rc.setGen(sum, i);
        }
        
        return (rastriginFunction.fitness(rc));
    }
    
    public double fitness(Chromosome c, Vector remaider){
        return (this.fitness(c));
    }
    
    /** returns a name for the funcion*/
    public String getName(){
        return "RRas";
    }
    
    /** return the dimension of the function*/
    public int getDimension(){
        return -1;
    }
}
