package geneticalgorithms.ChromosomeSelectors;

import FitnessFunctions.FitnessFunction;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import java.util.Vector;

/** This class implements a selection mechanism where the best chromosome is choose.
 * <p>Título: Genetic Algorithms</p>
 * <p>Descripción: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Empresa: </p>
 * @author Carlos García Martínez
 * @version 1.0
 */

public class FitnessBiasedChromosomeSelector implements ChromosomeSelector {
  
  FitnessFunction fitnessFunction;
  
  public FitnessBiasedChromosomeSelector(FitnessFunction fitnessFunction) {
    this.fitnessFunction = fitnessFunction;
  }
  
  /**
   * This function returns the position of the chromosome in population which has
   * the best fitness and it is not in set alreadySelected.
   *
   * @param population Population of chromosomes.
   * @param alreadySelected Set of selected chromosomes.
   * @return position of the new selected chromosome.
   */
  
  public int select(Vector population, Vector alreadySelected) {
    
    Vector aux = new Vector();
    aux.addAll(population);
    aux.removeAll(alreadySelected);
    double bestFitness;
    int numChromosomes, i, selected;
    Chromosome aChromosome;
    
    numChromosomes = aux.size();
    aChromosome = (Chromosome) aux.get(0);
    bestFitness = aChromosome.getCurrentFitness();
    selected = 0;
    
    for (i = 1; i < numChromosomes; i++){
      aChromosome = (Chromosome) aux.get(i);
      double currentFitness = aChromosome.getCurrentFitness();
      
      if (this.fitnessFunction.compare(currentFitness, bestFitness) > 0){
        selected = i;
        bestFitness = currentFitness;
      }
    }
    
    selected = population.indexOf(aux.get(selected));
    
    return (selected);
  }
}