package geneticalgorithms.ChromosomeSelectors;

import geneticalgorithms.DistanceFunctions.DistanceFunction;
import others.Random;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;

/**
 * <p>Título: Genetic Algorithms</p>
 * <p>Descripción: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Empresa: </p>
 * @author Carlos García Martínez
 * @version 1.0
 */

import java.util.Vector;

/**This class implements the NAM Selection mechanism.
 *@author Carlos García Martínez
 */

public class NamChromosomeSelector implements ChromosomeSelector {
  /** This object show how the distance between two chromosomes must be measured. */
  protected DistanceFunction distanceFunction;
  
  /** This variable indicates the size of the set with random chromosomes from the population. */
  protected int nass;
  
  /** nass >= 2*/
  public NamChromosomeSelector(DistanceFunction distanceFunction, int nass) {
    this.distanceFunction = distanceFunction;
    this.nass = nass;
  }
  
  /** This functions select a chromosome from the population. It takes into account the already selected ones
   * and returns the that chromosome more dissimilar to it.
   * It returns the position of the selected chromosome in the list population. */
  public int select(Vector population, Vector alreadySelected){
    
    int i, mostDissimilarPosition;
    int []auxiliarySet;
    int numElements = population.size() - 1;
    auxiliarySet = new int[this.nass];
    
    for (i = 0; i < this.nass; i++)
      auxiliarySet[i] = Random.Randint(0, numElements);
    
    double mostDissimilarity;
    Chromosome aChromosome = (Chromosome) population.get(auxiliarySet[0]);
    mostDissimilarity = this.distanceFunction.distance(aChromosome, alreadySelected);
    mostDissimilarPosition = 0;
    
    for (i = 1; i < this.nass; i++){
      aChromosome = (Chromosome) population.get(auxiliarySet[i]);
      double currentDissimilarity = this.distanceFunction.distance(aChromosome, alreadySelected);
      
      if (currentDissimilarity > mostDissimilarity){
        mostDissimilarity = currentDissimilarity;
        mostDissimilarPosition = i;
      }
    }
    
    return (auxiliarySet[mostDissimilarPosition]);
  }
}