package geneticalgorithms.DistanceFunctions;

import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import geneticalgorithms.ChromosomeRepresentations.RealChromosome;

/**
 * <p>Título: Genetic Algorithms</p>
 * <p>Descripción: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Empresa: </p>
 * @author Carlos García Martínez
 * @version 1.0
 */

import java.util.Vector;
import java.lang.Double;

/**
 * This class implements the euclidean distance function.
 * It works with objects of at least the class RealChromosome.
 */

public class EuclideanDistance implements DistanceFunction {
  
  //! Constructor
  public EuclideanDistance(){}
  
  /** This function returns the distance between two Chromosomes. */
  public double distance(Chromosome c1, Chromosome c2){
    
    int i;
    double sum;
    
    RealChromosome rc1 = (RealChromosome) c1;
    RealChromosome rc2 = (RealChromosome) c2;
    int numGenes = rc1.getNumGenes();
    sum = 0;
    
    for (i = 0; i < numGenes; i++){
      double currentDifference = rc1.getGen(i) - rc2.getGen(i);
      sum += (currentDifference * currentDifference);
    }
    
    return Math.sqrt(sum);
    
  }
  
  /** This function returns the distance between a chromosome and a set.
   * It returns the distance between c1 to the closest chromosome in set. */
  public double distance(Chromosome c1, Vector set){
    
    int i;
    int numChromosomes = set.size();
    
    if (numChromosomes <= 0)
      return Double.MAX_VALUE;
    
    RealChromosome aChromosome = (RealChromosome) set.get(0);
    double minDistance = this.distance(c1, aChromosome);
    
    for (i = 1; i < numChromosomes; i++){
      aChromosome = (RealChromosome) set.get(i);
      double currentDistance = this.distance(c1, aChromosome);
      
      if (currentDistance < minDistance)
        minDistance = currentDistance;
    }
    
    return minDistance;
  }
}