package geneticalgorithms.GAStructures;

import geneticalgorithms.Crossovers.Crossover;
import geneticalgorithms.Mutators.Mutator;
import FitnessFunctions.FitnessFunction;
import geneticalgorithms.ReplacementStrategies.ReplacementStrategy;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import geneticalgorithms.StopCriterions.StopCriterion;
import geneticalgorithms.Initializers.Initializer;
import geneticalgorithms.ChromosomeSelectors.ParentsSelector;
import others.UtilFunctions;
import geneticalgorithms.Reporters.Reporter;

/**
 * <p>Título: Genetic Algorithms</p>
 * <p>Descripción: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Empresa: </p>
 * @author Carlos García Martínez
 * @version 1.0
 */

import java.util.Vector;

/**This class implements the common behavior of a Sexual Steady State Genetic Algorithm.   */

public class S3GA extends SOGA{
  
  protected ParentsSelector parentsSelector;
  protected Crossover crossover;
  protected Mutator mutator;
  protected ReplacementStrategy replacer;
  
  /** Constructor. */
  public S3GA(FitnessFunction fitnessFunction, Initializer initializer, Crossover crossover, Mutator mutator,
       ParentsSelector parentsSelector, ReplacementStrategy replacer, StopCriterion stopCriterion,
       int numChromosomes, Reporter reporter){
    
    super(fitnessFunction, initializer, stopCriterion, numChromosomes, reporter);
    
    int i;
    this.crossover = crossover;
    this.mutator = mutator;
    this.parentsSelector = parentsSelector;
    this.replacer = replacer;
  }
  
  public S3GA(Vector population, FitnessFunction fitnessFunction, Crossover crossover, Mutator mutator,
       ParentsSelector parentsSelector, ReplacementStrategy replacer, StopCriterion stopCriterion,
       Reporter reporter){
    
    super(fitnessFunction, population, stopCriterion, reporter);
    
    int i;
    this.crossover = crossover;
    this.mutator = mutator;
    this.parentsSelector = parentsSelector;
    this.replacer = replacer;
  }
  
  /** This functions performs an interation. */
  public void iterate(){
    
    int i;
    
    //Parent Selection
    Vector positionParents = new Vector();
    Vector selectedParents = new Vector();
    this.parentsSelector.select(positionParents, selectedParents, this.population);
    
    //Crossing
    Vector offspring = new Vector();
    this.crossover.cross(offspring, selectedParents);
    this.mutator.mutate(offspring);
    
    //Fitness assigning
    int numOffspring = offspring.size();
    Chromosome anOffspring;
    
    for (i = 0; i < numOffspring; i++){
      anOffspring = (Chromosome) offspring.get(i);
      
      anOffspring.setCurrentFitness(this.fitnessFunction.fitness(anOffspring, this.population));
    }
    
    //Replacement strategy
    this.replacer.performReplacement(this, positionParents, offspring);
  }
}