package geneticalgorithms.GAStructures;

import geneticalgorithms.Crossovers.Crossover;
import geneticalgorithms.Mutators.*;
import geneticalgorithms.ChromosomeRepresentations.UFSRealChromosome;
import FitnessFunctions.FitnessFunction;
import geneticalgorithms.ReplacementStrategies.ReplacementStrategy;
import geneticalgorithms.ChromosomeRepresentations.Chromosome;
import geneticalgorithms.StopCriterions.StopCriterion;
import geneticalgorithms.Initializers.Initializer;
import geneticalgorithms.ChromosomeSelectors.ParentsSelector;
import others.UtilFunctions;
import geneticalgorithms.Reporters.Reporter;
import others.Random;
import java.util.Vector;

/**
 * <p>Título: Genetic Algorithms</p>
 * <p>Descripción: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Empresa: </p>
 * @author Carlos García Martínez
 * @version 1.0
 */


public class SCDGA extends S3GA {
  
  /** Number of female chromosomes. */
  protected int Nf;
  
  /** Number of male chromosomes. */
  protected int Nm;
  
  /** This variable indicates the way the algorithm select the female chromosomes initially. */
  ParentsSelector femaleInitialSelector;
  
  /** This variable indicates the way the algorithm select the male chromosomes initially. */
  ParentsSelector maleInitialSelector;
  
  /** Vector to the female chromosomes. */
  protected Vector femaleChromosomes;
  
  /** Vector of the male chromosomes. */
  Vector maleChromosomes;
  
  /** Replacement strategy. */
  ReplacementStrategy replacer;
  
  /** Selection rule for the female parent. */
  ParentsSelector femaleParentSelector;
  
  /** Selection rule for the male parents. */
  ParentsSelector maleParentsSelector;
  
  public SCDGA(FitnessFunction fitnessFunction, Initializer initializer,
       ParentsSelector femaleInitialSelector, ParentsSelector maleInitialSelector,
       Crossover crossover, Mutator mutator,
       ParentsSelector femaleParentSelector,
       ParentsSelector maleParentsSelector, ReplacementStrategy replacer,
       StopCriterion stopCriterion, Reporter reporter, int Nf, int Nm) {
    
    super(fitnessFunction, initializer, crossover, mutator, maleParentsSelector,
         replacer, stopCriterion, Math.max(Nf,Nm), reporter);
    
    int i;
    Chromosome aChromosome;
    Vector aux = new Vector();
    Vector none = new Vector();
    
    this.Nf = Nf;
    this.Nm = Nm;
    this.replacer = replacer;
    this.femaleParentSelector = femaleParentSelector;
    this.maleParentsSelector = maleParentsSelector;
    this.maleChromosomes = new Vector();
    this.femaleChromosomes = new Vector();
    this.femaleInitialSelector = femaleInitialSelector;
    this.maleInitialSelector = maleInitialSelector;
    
    if (Nf < Nm){
      this.maleChromosomes.addAll(this.population);
      this.femaleInitialSelector.select(none, this.femaleChromosomes, this.population);
    }
    
    else{
      this.femaleChromosomes.addAll(population);
      this.maleInitialSelector.select(none, this.maleChromosomes, this.population);
    }
  }
  
  /**
   * @param femalePopulation
   * @param malePopulation
   * @param fitnessFunction
   * @param crossover
   * @param femaleParentSelector
   * @param maleParentsSelector
   * @param femaleReplacer
   * @param maleReplacer
   * @param stopCriterion
   */
  
  public SCDGA(Vector newPopulation, ParentsSelector femaleInitialSelector,
       ParentsSelector maleInitialSelector,
       FitnessFunction fitnessFunction,
       Crossover crossover, Mutator mutator,
       ParentsSelector femaleParentSelector,
       ParentsSelector maleParentsSelector, ReplacementStrategy replacer,
       StopCriterion stopCriterion, Reporter reporter){
    
    super(newPopulation, fitnessFunction, crossover, mutator, maleParentsSelector,
         replacer, stopCriterion, reporter);
    
    Vector none = new Vector();
    this.femaleInitialSelector = femaleInitialSelector;
    this.maleInitialSelector = maleInitialSelector;
    this.femaleChromosomes = new Vector();
    this.maleChromosomes = new Vector();
    this.femaleInitialSelector.select(none, this.femaleChromosomes, this.population);
    this.maleInitialSelector.select(none, this.maleChromosomes, this.population);
    this.Nf = this.femaleChromosomes.size();
    this.Nm = this.maleChromosomes.size();
    this.population.removeAllElements();
    
    if (this.Nf < this.Nm){
      this.maleChromosomes.removeAll(this.femaleChromosomes);
      this.maleChromosomes.addAll(this.femaleChromosomes);
      this.population.addAll(this.maleChromosomes);
    } else{
      this.femaleChromosomes.removeAll(this.maleChromosomes);
      this.femaleChromosomes.addAll(this.maleChromosomes);
      this.population.addAll(this.femaleChromosomes);
    }
    
    this.Nf = this.femaleChromosomes.size();
    this.Nm = this.maleChromosomes.size();
    this.numChromosomes = population.size();
    this.replacer = replacer;
    this.femaleParentSelector = femaleParentSelector;
    this.maleParentsSelector = maleParentsSelector;
  }
  
  
  /** This function run the GA until the Stop Criterion is reached. */
  public void run(){
    
    numIterations = 0;
    
    while (!this.stopCriterion.isTrue(this)){
      
      if (reporter != null && reporter.needReport(this))
        reporter.report(this);
      
      this.iterate();
      numIterations++;
    }
  }
  
  
  /** This functions performs an interation. */
  public void iterate(){
    
    int i, numOffspring;
    Vector selectedParents = new Vector();
    Chromosome aParent, aChromosome;
    Vector offspring = new Vector();
    
    Vector positionParents = new Vector();
    
    //Parent Selection
    this.femaleParentSelector.select(positionParents, selectedParents, this.femaleChromosomes);
    this.maleParentsSelector.select(positionParents, selectedParents, this.maleChromosomes);
    
    //Crossing
    this.crossover.cross(offspring, selectedParents);
    
    //Mutate
    this.mutator.mutate(offspring);
    
    //Fitness assigning
    numOffspring = offspring.size();
    
    for (i = 0; i < numOffspring; i++){
      aChromosome = (Chromosome) offspring.get(i);
      double newFitness = this.fitnessFunction.fitness(aChromosome);
      aChromosome.setCurrentFitness(newFitness);
    }
    
    this.replacer.performReplacement(this, positionParents, offspring);
  }
  
  public Vector getFemalePopulation(){
    return (this.femaleChromosomes);
  }
  
  public Vector getMalePopulation(){
    return (this.maleChromosomes);
  }
  
  public void setStopCriterion(StopCriterion stopC) {
    
    this.stopCriterion = stopC;
  }
}