/*
 * GlobalLocal.java
 *
 * Created on 8 de abril de 2007, 21:14
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package geneticalgorithms;
import geneticalgorithms.Initializers.*;
import FitnessFunctions.RealFF.RealFitnessFunction;
import geneticalgorithms.Replacers.*;
import geneticalgorithms.ReplacementStrategies.*;
import geneticalgorithms.ChromosomeSelectors.*;
import geneticalgorithms.Crossovers.*;
import geneticalgorithms.Mutators.*;
import geneticalgorithms.Crossovers.RealCrossovers.PBXalphaCrossover;
import geneticalgorithms.StopCriterions.*;
import geneticalgorithms.Reporters.*;
import others.Random;
import geneticalgorithms.GAStructures.*;
import geneticalgorithms.DistanceFunctions.*;
import geneticalgorithms.ChromosomeRepresentations.*;

/**
 *
 * @author carlos
 */
public class GlobalLocal {
  
  Initializer initializer;
  RealFitnessFunction fitnessFunction;
  int numFemales1, numMales1, numFemales2, numMales2;
  double porcentajeGlobal;
  long maxEvaluations;
  Random random;
  UFSChromosomeSelector femaleSelector;
  ParentsSelector femaleParentSelector;
  EuclideanDistance distanceFunction;
  NamChromosomeSelector maleSelector;
  ParentsSelector maleParentSelector;
  
  /** Creates a new instance of GlobalLocal */
  public GlobalLocal(Initializer initializer, RealFitnessFunction fitnessFunction,
       Random random, long maxEvaluations) {
    this.initializer = initializer;
    this.fitnessFunction = fitnessFunction;
    this.maxEvaluations = maxEvaluations;
    this.random = random;
    numFemales1 = 200;
    numMales1 = 400;
    numFemales2 = 5;
    numMales2 = 100;
    porcentajeGlobal = 0.25;
    this.femaleSelector = new UFSChromosomeSelector();
    this.femaleParentSelector = new ParentsSelector(1, femaleSelector);
    this.distanceFunction = new EuclideanDistance();
    this.maleSelector = new NamChromosomeSelector(distanceFunction, 5);
    this.maleParentSelector = new ParentsSelector(1, maleSelector);
  }
  
  public Chromosome run(){
    
    /********************
     *GLOBAL GA
     ********************/
    Replacer femaleReplacer = new ReplaceWorst(fitnessFunction);
    Replacer maleReplacer = new ReplaceWorst(fitnessFunction);
    ReplacementStrategy replacer = new RestrictedThenRemainderStrategy(femaleReplacer, maleReplacer);
    ChromosomeSelector diffStrategy = new
         FitnessBiasedChromosomeSelector(fitnessFunction);
    ParentsSelector femaleInitialSelector = new ParentsSelector(numFemales1, diffStrategy);
    ParentsSelector maleInitialSelector = new ParentsSelector(numMales1, diffStrategy);
    Crossover crossover = new PBXalphaCrossover(0.8, fitnessFunction);
    Mutator mutator = new NoneMutator();
    NumEvaluationsCriterion stopCriterion = new NumEvaluationsCriterion((long)(maxEvaluations * 0.25));
    Reporter aReporter = null;
    
    SCDGA ga = new SCDGA(fitnessFunction, initializer,
         femaleInitialSelector,
         maleInitialSelector,
         crossover, mutator,
         femaleParentSelector, maleParentSelector,
         replacer, stopCriterion, aReporter, numFemales1, numMales1);
    ga.run();
    
    /********************
     *LOCAL GA
     *******************/
    stopCriterion.setMaxEvaluations(maxEvaluations);
    ParentsSelector femaleInitialSelector2 = new ParentsSelector(numFemales2, diffStrategy);
    ParentsSelector maleInitialSelector2 = new ParentsSelector(numMales2, diffStrategy);
    ChromosomeSelector femaleSelector2 = femaleSelector;
    ParentsSelector femaleParentSelector2 = new ParentsSelector(1, femaleSelector2);
    ChromosomeSelector maleSelector2 = maleSelector;
    ParentsSelector maleParentsSelector2 = new ParentsSelector(2, maleSelector2);
    
    SCDGA ga2 = new SCDGA(ga.getPopulation(),
         femaleInitialSelector2,
         maleInitialSelector2,
         fitnessFunction,
         crossover, mutator,
         femaleParentSelector2, maleParentsSelector2,
         replacer, stopCriterion, aReporter);
    
    ga2.run();
    
    if (aReporter != null)
      aReporter.close(ga2);
    
    return (ga2.getBestChromosome());
  }
  
}
