/*
 * UniformInitializer.java
 *
 * Created on 26 de febrero de 2005, 18:59
 */

package geneticalgorithms.Initializers;
import geneticalgorithms.ChromosomeRepresentations.RealChromosome;
import java.util.Vector;
import others.Random;

/**
 *
 * @author Carlos
 */
public class UniformInitializer implements Initializer{
  
  /** Indicates the minimum of the uniform random initialisation for every gen. */
  protected double min[];
  
  /** Indicates the maximum of the uniform random initialisation for every gen. */
  protected double max[];
  
  /** Template used for creating the rest of the chromosomes*/
  protected RealChromosome templateChromosome;
  
  /** Creates a new instance of UniformInitializer */
  public UniformInitializer(RealChromosome templateChromosome, double min, double max) {
    this.templateChromosome = templateChromosome;
    int numGenes = templateChromosome.getNumGenes();
    this.min = new double[numGenes];
    this.max = new double[numGenes];
    
    int i;
    
    for (i = 0; i < numGenes; i++){
      this.min[i] = min;
      this.max[i] = max;
    }
  }
  
  public void setIthBound(int index, double min, double max){
    this.min[index] = min;
    this.max[index] = max;
  }
  
  public void initialize(Vector chromosomes, int numChromosomes) {
    
    int i, j;
    int numGenes;
    
    chromosomes.removeAllElements();
    numGenes = this.templateChromosome.getNumGenes();
    
    for (i = 0; i < numChromosomes; i++){
      
      RealChromosome aChromosome = (RealChromosome) this.templateChromosome.clone();
      
      for (j = 0; j < numGenes; j++)
        aChromosome.setGen(Random.Randfloat(this.min[j], this.max[j]), j);
      
      chromosomes.add(aChromosome);
    }
  }
}
