/*
 * Program.java
 *
 * Created on 23 de enero de 2005, 22:40
 */

package geneticalgorithms;
import geneticalgorithms.ChromosomeRepresentations.*;
import geneticalgorithms.GlobalLocal;
import geneticalgorithms.Initializers.*;
import java.util.Vector;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.io.ObjectOutputStream;
import FitnessFunctions.RealFF.ScaledRastriginFitnessFunction;
import FitnessFunctions.RealFF.RotatedRastriginFitnessFunction;
import FitnessFunctions.RealFF.Schwefel2_21FitnessFunction;
import FitnessFunctions.RealFF.Schwefel1_2FitnessFunction;
import FitnessFunctions.RealFF.SchafferFitnessFunction;
import FitnessFunctions.RealFF.GriewangkFitnessFunction;
import FitnessFunctions.RealFF.RastriginFitnessFunction;
import FitnessFunctions.RealFF.CigarTabletFitnessFunction;
import FitnessFunctions.RealFF.Schwefel2_22FitnessFunction;
import FitnessFunctions.RealFF.ComposedGriRos;
import FitnessFunctions.RealFF.QuarticNoiseFitnessFunction;
import FitnessFunctions.RealFF.EF10;
import FitnessFunctions.RealFF.LinearEquationsSystemFitnessFunction;
import FitnessFunctions.RealFF.BohachevskyFitnessFunction;
import FitnessFunctions.RealFF.RealFitnessFunction;
import FitnessFunctions.RealFF.RosenbrockFitnessFunction;
import FitnessFunctions.RealFF.SphereFitnessFunction;
import FitnessFunctions.RealFF.Schwefel2_26FitnessFunction;
import FitnessFunctions.FitnessFunction;
import others.UtilFunctions;
import others.Parameters;
import others.Random;

/**
 *
 * @author Carlos
 */
public class Program{
  
  
  /** run the program */
  public static void main(String[] args){
    
    int i=0, j=0, k=0, l=0, m=0, n=0;
    int numFitnessFunctions;
    int numEjecuciones = 2;
    Random random = new Random();
    Vector initializers = new Vector();
    Vector fitnessFunctions = new Vector();
    String[] fitnessFunctionNames = {"sph", "Ros", "Sch1.2", "Ras", "Gri", "Psle", "Boh", "Cig", "Sch2_22", "Sch2_21", "QNoise", "RRas", "SRas", "Sch2_26", "EF10", "F8F2", "Schaffer"};
    
    //Fitness functions
    fitnessFunctions.add(new SphereFitnessFunction(-5.12, 5.12));
    fitnessFunctions.add(new RosenbrockFitnessFunction(-5.12, 5.12));
    fitnessFunctions.add(new Schwefel1_2FitnessFunction(-65.536, 65.536));
    fitnessFunctions.add(new RastriginFitnessFunction(10, 2 * Math.PI, -5.12, 5.12));
    fitnessFunctions.add(new GriewangkFitnessFunction(4000, -600, 600));
    fitnessFunctions.add(new LinearEquationsSystemFitnessFunction(-127, 127));
    fitnessFunctions.add(new BohachevskyFitnessFunction(-15, 15));
    fitnessFunctions.add(new CigarTabletFitnessFunction(-7, 7));
    fitnessFunctions.add(new Schwefel2_22FitnessFunction(-10, 10));
    fitnessFunctions.add(new Schwefel2_21FitnessFunction(-100, 100));
    fitnessFunctions.add(new QuarticNoiseFitnessFunction(-1.28, 1.28));
    fitnessFunctions.add(new RotatedRastriginFitnessFunction(25, -5.12, 5.12));
    fitnessFunctions.add(new ScaledRastriginFitnessFunction(-5.12, 5.12));
    fitnessFunctions.add(new Schwefel2_26FitnessFunction(-500, 500));
    fitnessFunctions.add(new EF10(-100, 100));
    fitnessFunctions.add(new ComposedGriRos(-5, 5));
    fitnessFunctions.add(new SchafferFitnessFunction(-100, 100));
    initializers.add(new UniformUFSRealInitializer(25, 4, 5));
    initializers.add(new UniformUFSRealInitializer(25, -5, -4));
    initializers.add(new UniformUFSRealInitializer(25, 60, 65));
    initializers.add(new UniformUFSRealInitializer(25, 4, 5));
    initializers.add(new UniformUFSRealInitializer(15, 580, 600));
    initializers.add(new UniformUFSRealInitializer(10, -120, -100));
    initializers.add(new UniformUFSRealInitializer(25, 10, 15));
    initializers.add(new UniformUFSRealInitializer(25, 5, 7));
    initializers.add(new UniformUFSRealInitializer(25, 8, 10));
    initializers.add(new UniformUFSRealInitializer(100, 80, 100));
    initializers.add(new UniformUFSRealInitializer(25, -1.28, 1.28));
    initializers.add(new UniformUFSRealInitializer(25, 4, 5));
    initializers.add(new UniformUFSRealInitializer(25, 4, 5));
    initializers.add(new UniformUFSRealInitializer(30, -500, 300));
    initializers.add(new UniformUFSRealInitializer(15, -100, 100));
    initializers.add(new UniformUFSRealInitializer(10, -5, 5));
    initializers.add(new UniformUFSRealInitializer(25, 50, 100));
    numFitnessFunctions = fitnessFunctions.size();
    
    
    for (int k2 = 0; k2 < numFitnessFunctions; k2++){
      
      if (k2 != 0)
        System.out.print("\t" + fitnessFunctionNames[k2]);
      else
        System.out.print(fitnessFunctionNames[k2]);
    }
    
    System.out.println();
    
    for (l = 0; l < numEjecuciones; l++) {
      
      for (k = 0; k < numFitnessFunctions; k++) {
        
        Initializer initializer = (Initializer) initializers.get(k);
        RealFitnessFunction fitnessFunction = (RealFitnessFunction)
        fitnessFunctions.get(k);
        fitnessFunction.resetNumEvaluations();
        random.cambiaSemilla(Parameters.semillas[l]);
        
        /*********************/
        /*********************/
        GlobalLocal optimizer = new GlobalLocal(initializer, fitnessFunction,
             random, Parameters.numEvaluations);
        Chromosome solution = optimizer.run();
        /*********************/
        /*********************/
        
        if (k != 0)
          System.out.print("\t" + solution.getCurrentFitness());
        else
          System.out.print(solution.getCurrentFitness());
      }
      
      System.out.println();
    }
  }
}
